/***************************************************************************
                          screen.cpp  -  description
                             -------------------
    begin                : Tue Dec 10 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "screen.h"

#include <klocale.h>

#include "cdp.h"
#include "movingobject.h"
#include "state.h"
#include "event.h"
#include "timer.h"
#include "cachablepixmaparray.h"

Screen::Screen():QCanvasSprite(0, TPBase::canvas) {
	num_objects = num_states = num_events = num_timers = 0;
	image = 0;
	background = 0;
}

Screen::~Screen(){
	unsigned int i;
	if (num_objects) {
		for (i = 0; i < num_objects; i++) {
			delete objects[i];
		}
		delete[]objects;
	}

	if (cdp->for_real () || cdp->thorough_check ()) {
		hide ();
		if (image) CachablePixmapArray::discard (image);
		TPBase::canvas->setBackgroundColor (QColor (0, 0, 0));
		if (background) delete background;
	}

	if (num_states) {
		for (i = 0; i < num_states; i++) {
			delete states[i];
		}
		delete[]states;
	}

	if (num_events) {
		for (i = 0; i < num_events; i++) {
			delete events[i];
		}
		delete[]events;
	}

	if (num_timers) {
		for (i = 0; i < num_timers; i++) {
			delete timers[i];
		}
		delete[]timers;
	}
}

/** Returns a pointer to the state identified with ident. Must never be called while not within read_level () ! */
State *Screen::resolve_state (QString & ident) {
	if (state_map.find (ident) == state_map.end ()) {
		cdp->debug_msg ( i18n ( "State '%1' referenced but undeclared!" ).arg(ident), Critical);
		return 0;
	} else {
		return (state_map[ident]);
	}
}

/** Returns a pointer to the event identified with ident. Must never be called while not within read_level () ! */
Event *Screen::resolve_event (QString & ident) {
	if (event_map.find (ident) == event_map.end ()) {
		cdp->debug_msg ( i18n ( "Event '%1' referenced but undeclared!" ).arg(ident), Critical);
		return 0;
	} else {
		return (event_map[ident]);
	}
}

/** Reads config for this screen (basically everything common to levels
	and splashes) */
void Screen::readScreen(const QDomElement & xdocElement) {
	QString convert;			// used to convert numbers
	QDomElement e;				// just some element
	QDomNodeList list;			// just a list of nodes

	caption = cdp->get_string_attribute("caption", xdocElement, i18n("No Title"), Warn);
	author = cdp->get_string_attribute("author", xdocElement, i18n("Anonymous"), Info);

	if (! (cdp->for_real() || cdp->thorough_check())) { // all the rest not needed for credits
		return;
	}

	e = cdp->get_element("image", xdocElement, Critical);	// get image-section
	background = cdp->createQCPixmap ("background", e, 640, 480, Serious);
	int i = 1;
	image = cdp->createPixmapArray("foreground", e, &i, 640, 480, cdp->get_bool_attribute("alpha", e, false, Info));

// BEGIN Read states, events and timers here!

// get state-declarations
	e = cdp->get_element ("states", xdocElement, Info);	// get all the states

	if (!(e.isNull())) {		// found a states-section?
		list = cdp->get_node_list ("state", e, 1, -1, 1, Warn);

		if (list.length ()) {
			states = new State * [list.length()];

			while (num_states < list.length ()) {
				e = list.item (num_states).toElement ();

				states[num_states] = new State (&e);
				state_map.insert (states[num_states]->getIdent (), states[num_states]);

	        		num_states++;
			}
		}
	}

// read events
	e = cdp->get_element ("events", xdocElement, Info);	// get all the events

	if (!(e.isNull())) {		// found an events-section?
		list = cdp->get_node_list ("event", e, 1, -1, 1, Warn);

		if (list.length ()) {
			events = new Event * [list.length()];

			while (num_events < list.length ()) {
				e = list.item(num_events).toElement ();

				events[num_events] = new Event (&e);
				event_map.insert (events[num_events]->getIdent (), events[num_events]);

	        		num_events++;
			}
		}
	}

// read timers
	e = cdp->get_element ("timers", xdocElement, Info);	// get all the timers

	if (!(e.isNull())) {		// found an timers-section?
		list = cdp->get_node_list ("timers", e, 1, -1, 1, Warn);

		if (list.length ()) {
			timers = new Timer * [list.length()];

			while (num_timers < list.length ()) {
				e = list.item(num_timers).toElement ();

				timers[num_timers] = new Timer (&e);
				event_map.insert (timers[num_timers]->getIdent (), timers[num_timers]);

	        		num_timers++;
			}
		}
	}


// now resolve the child-events in all events and timers
	for (unsigned int i=0; i < num_events; i++) {
		events[i]->resolve_now ();
	}
	for (unsigned int i=0; i < num_timers; i++) {
		timers[i]->resolve_now ();
	}

// END: events, states, timers

	e = cdp->get_element("objects", xdocElement, Info);	// now get the MovingObjects

	if (!(e.isNull())) {		// did we really find an objects-section?

		list = cdp->get_node_list("object", e, 1, -1, 1, Warn);	// get each single object in the section

		if (list.length ()) {
			objects = new MovingObject * [list.length()];

			while (num_objects < list.length ()) {				// create this object
				e = list.item (num_objects).toElement ();

				objects[num_objects] = new MovingObject ( &e );

				num_objects++;
			}
		}
	}
}

/** Clears the state- and event-maps (to be called after all entities were read) */
void Screen::clearMaps () {
	state_map.clear ();				// do not call resolve_state () after this point!
	event_map.clear ();				// do not call resolve_event () after this point!
}
