/***************************************************************************
                          timemanager.h  -  description
                             -------------------
    begin                : Sun Dec 23 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TIMEMANAGER_H
#define TIMEMANAGER_H

#include <qobject.h>
#include <stdint.h>
#include <qlist.h>

#include "timedobject.h"
#include "tpbase.h"

/** This class provides time-management similar to that of QObject-timers.

The advantages are:
* a (probably) better speed-efficiency
* the ability to group sets of timers in an instance of a TimeManager and to
* pause and resume all timers in such a group, without losing be current timers' states
	(e.g. whether timer x is already close to firing). Very helpful, if e.g. you want to pause a game
* the ability set a timer that always gets fired last in a round (use this e.g. for redrawing the screen, after all items
	that needed to be updated have been updated).
* no need to derive classes that need timers from (expensive) QObject
* timer-leaks can't grow for ever (because - see disadvantages - the number of available timers is limited) and you
	will be warned, if your app tries to allocate more timers than are available

The disadvantages are:
* a (probably) worse memory-efficiency (depending on the size of some data-types on your system
	somewhere around 20 (8bit) bytes per available (not per actually used) timer, but remember that you save inheriting QObject)
* the need to specify in advance, how many timers you will need max (keeping in mind the above point)
* a fixed maximum timer-resolution for all timers in the TimeManager (parameter base_intervall in the constructor; errors should not be accumulating)
* the need to derive classes that need timers from TimedObject
* less sanity-checking (esp. all TimedObjects using this TimeManager share the same pool of timer-ids and
	therefore could potentially mess with each other's timers - so you need to code carefully)

  *@author Thomas Friedrichsmeier
  */

class TimedObject;

class TimeManager : public QObject, public TPBase  {
public: 
/** Sets up a TimeManger for at most num_timers timers, with an internal resolution corresponding to base_intervall msecs */
	TimeManager(int num_timers, int base_intervall);
/** Discards the TimeManager */
	~TimeManager();
/** Starts a timer for TimedObject to with interval timer_period msec, returning id assigned to this timer.
	Returned ids are always in the range of 0 .. max_timers -1 */
	int startTimer (TimedObject *to, int_fast32_t timer_period);
/** Dirty hack: set a timer to be checked at the very end of the list (to be used, if you want a certain thing to be done after all
	other things, like redrawing the screen, after all sprites have been updated). No sanity checks. Any previous timer at this
	place gets deleted without notice! */
	int setLastTimer (TimedObject *to, int_fast32_t timer_period);
/** Kills a timer by its id. Warning, will not check, whether the caller really "owns" that timer. Passing an id of a timer that is not
	in use in any timed object will not do any harm, however. */
	void killTimer (int id);
/** Kills all timers for a specified object. Does not kill the special last timer */
	void killTimers (TimedObject *to);
/** Kills the special last timer */
	void killLastTimer ();
/** Resumes all timers */
	void resume ();
/** Pauses ALL timers */
	void pause ();

protected:					// Protected methods
/** This catches the ticks of the actual timer */
	void timerEvent(QTimerEvent * e);

private:
/** All the internal information about the active timers */
	struct Timer {
		TimedObject *to;
		int_fast32_t increase;
		int_fast32_t score;
	} *timers;
/** Stores a list of both free (not yet used) and used timer-ids. Used timer ids are always below index active_timers in this array,
	while used ids are at and above that index position. */
	int *ids;
/** How many timers there are currently */
	int active_timers;
/** Maximum number of timers available (not including the "last timer"). See constructor */
	int max_timers;
/** Internal resolution. See constructor */
	int base_ti;
};

#endif
